#ifndef GTB_LINE3_INCLUDED
#define GTB_LINE3_INCLUDED

#include "gtbmousecommon.h"
#include "point3.hpp"
#include "vector3.hpp"

BEGIN_GTBMOUSENS


class Ray3;


class Line3 {
public:
	Line3();
	Line3(const Line3 &l);
	Line3(const Point3 &p, const Point3 &q);
	Line3(const Point3 &p, const Vector3 &d);
	explicit Line3(const Ray3 &r);
	Line3 &operator=(const Line3 &r);

	bool operator==(const Line3 &l) const;
	bool operator!=(const Line3 &l) const;

	Line3 operator-() const;

	const Vector3 &direction() const;
	bool is_degenerate() const;

	// Returns an arbitrary point on the line.
	// point(i) == point(j), iff i == j.
	Point3 point(int i) const;

	bool contains(const Point3 &p) const;
	Point3 projection(const Point3 &p) const;

protected:
	Point3 _p;
        Vector3 _d;
};


extern const Line3 LINE3_POSITIVE_X;
extern const Line3 LINE3_NEGATIVE_X;
extern const Line3 LINE3_POSITIVE_Y;
extern const Line3 LINE3_NEGATIVE_Y;
extern const Line3 LINE3_POSITIVE_Z;
extern const Line3 LINE3_NEGATIVE_Z;


END_GTBMOUSENS

#ifndef OUTLINE
#include "line3.ipp"
#endif

#endif // GTB_LINE3_INCLUDED
