#ifndef GTB_POINT2_INCLUDED
#define GTB_POINT2_INCLUDED

#include <vector>
#include "gtbmousecommon.h"

BEGIN_GTBMOUSENS


class Vector2;


class Point2 {
public:
	Point2();
	Point2(const Point2 &p);
	Point2(real_t px, real_t py);
	Point2(const real_t p[2]);
	Point2 &operator=(const Point2 &p);

	real_t x() const;
	real_t y() const;

	void set_x(real_t px);
	void set_y(real_t py);
	Point2 &reset(real_t px, real_t py);

	real_t operator[](unsigned i) const;
	real_t &operator[](unsigned i);

	bool operator==(const Point2 &p) const;
	bool operator!=(const Point2 &p) const;
	bool is_zero() const;

	Point2 &operator+=(const Vector2 &v);
	Point2 &operator-=(const Vector2 &v);

	friend Point2 operator+(const Point2 &p, const Vector2 &v);
	friend Vector2 operator-(const Point2 &p, const Point2 &q);
	friend Point2 operator-(const Point2 &p, const Vector2 &v);

	Point2 &scale(const Point2 &origin, real_t s);
	Point2 &translate(const Vector2 &v);
	Point2 &translate(real_t dx, real_t dy);
	Point2 &rotate(real_t theta);
	Point2 &rotate(const Point2 &origin, real_t theta);

	static real_t distance(const Point2 &p, const Point2 &q);
	static real_t squared_distance(const Point2 &p, const Point2 &q);

	static Point2 midpoint(const Point2 &A, const Point2 &B);
	static Point2 centroid(const std::vector<Point2> &v);

	void load() const;

  	friend std::istream &operator>>(std::istream &s, Point2 &p);
  	friend std::ostream &operator<<(std::ostream &s, const Point2 &p);

protected:
	real_t _p[2];
};


extern const Point2 POINT2_ZERO;


END_GTBMOUSENS

#ifndef OUTLINE
#include "point2.ipp"
#endif

#endif // GTB_POINT2_INCLUDED
