#include <GL/gl.h>
#include "vector3.hpp"
//#include <gtb/io/io.hpp>


BEGIN_GTBMOUSENS


inline Point3::Point3()
{
	_p[0] = 0.0;
	_p[1] = 0.0;
	_p[2] = 0.0;
}


inline Point3::Point3(const Point3 &p)
{
	_p[0] = p._p[0];
	_p[1] = p._p[1];
	_p[2] = p._p[2];
}


inline Point3::Point3(real_t px, real_t py, real_t pz)
{
	_p[0] = px;
	_p[1] = py;
	_p[2] = pz;
}


inline Point3::Point3(const real_t p[3])
{
	_p[0] = p[0];
	_p[1] = p[1];
	_p[2] = p[2];
}


inline Point3 &Point3::operator=(const Point3 &p)
{
	_p[0] = p._p[0];
	_p[1] = p._p[1];
	_p[2] = p._p[2];
	return *this;
}


inline bool Point3::operator==(const Point3 &p) const
{
#if 0
	return (real::is_equal(_p[0], p._p[0]) &&
		real::is_equal(_p[1], p._p[1]) &&
		real::is_equal(_p[2], p._p[2]));
#else
    return false;
#endif
}


inline bool Point3::operator!=(const Point3 &p) const
{
	return !(*this == p);
}


inline bool Point3::is_zero() const
{
#if 0
	return (real::is_zero(_p[0]) &&
		real::is_zero(_p[1]) &&
		real::is_zero(_p[2]));
#else
return false;
#endif
}


inline real_t Point3::x() const
{
	return _p[0];
}


inline real_t Point3::y() const
{
	return _p[1];
}


inline real_t Point3::z() const
{
	return _p[2];
}


inline void Point3::set_x(real_t px)
{
	_p[0] = px;
}


inline void Point3::set_y(real_t py)
{
	_p[1] = py;
}


inline void Point3::set_z(real_t pz)
{
	_p[2] = pz;
}


inline Point3 &Point3::reset(real_t px, real_t py, real_t pz)
{
	_p[0] = px;
	_p[1] = py;
	_p[2] = pz;
	return *this;
}


inline real_t Point3::operator[](unsigned i) const
{
	assert(i < 3);
	return _p[i];
}


inline real_t &Point3::operator[](unsigned i)
{
	assert(i < 3);
	return _p[i];
}


inline Point3 &Point3::operator*=(const Matrix4 &m)
{
	real_t px, py, pz, pw;
	px = x() * m[0][0] + y() * m[0][1] + z() * m[0][2] + m[0][3];
	py = x() * m[1][0] + y() * m[1][1] + z() * m[1][2] + m[1][3];
	pz = x() * m[2][0] + y() * m[2][1] + z() * m[2][2] + m[2][3];
	pw = x() * m[3][0] + y() * m[3][1] + z() * m[3][2] + m[3][3];
//	assert(!real::is_zero(pw));
	return reset(px / pw, py / pw, pz / pw);
}


inline Point3 &Point3::operator+=(const Vector3 &v)
{
	_p[0] += v._v[0];
	_p[1] += v._v[1];
	_p[2] += v._v[2];
	return *this;
}


inline Point3 &Point3::operator-=(const Vector3 &v)
{
	_p[0] -= v._v[0];
	_p[1] -= v._v[1];
	_p[2] -= v._v[2];
	return *this;
}

// Like += with with ponit parameter
inline void Point3::add(const Point3& p)
{
	_p[0] += p._p[0];
	_p[1] += p._p[1];
	_p[2] += p._p[2];
}

inline void Point3::scalar_scale(double s)
{
	_p[0] *= s;
	_p[1] *= s;
	_p[2] *= s;
}

inline Point3 Point3::scalar_scaled(double s) const
{
    Point3 r(_p[0] * s, _p[1] * s, _p[2] * s);
    return r;
}

inline Point3 operator*(const Matrix4 &m, const Point3 &p)
{
	real_t px, py, pz, pw;
	px = p.x() * m[0][0] + p.y() * m[0][1] + p.z() * m[0][2] + m[0][3];
	py = p.x() * m[1][0] + p.y() * m[1][1] + p.z() * m[1][2] + m[1][3];
	pz = p.x() * m[2][0] + p.y() * m[2][1] + p.z() * m[2][2] + m[2][3];
	pw = p.x() * m[3][0] + p.y() * m[3][1] + p.z() * m[3][2] + m[3][3];
//	assert(!real::is_zero(pw));
	return Point3(px / pw, py / pw, pz / pw);
}


inline Point3 operator+(const Point3 &p, const Vector3 &v)
{
	return Point3(p._p[0] + v._v[0],
		      p._p[1] + v._v[1],
		      p._p[2] + v._v[2]);
}


inline Vector3 operator-(const Point3 &p, const Point3 &q)
{
	return Vector3(p._p[0] - q._p[0],
		       p._p[1] - q._p[1],
		       p._p[2] - q._p[2]);
}


inline Point3 operator-(const Point3 &p, const Vector3 &v)
{
	return Point3(p._p[0] - v._v[0],
		      p._p[1] - v._v[1],
		      p._p[2] - v._v[2]);
}


inline Point3 &Point3::translate(const Vector3 &t)
{
	return *this += t;
}


inline Point3 &Point3::translate(real_t dx, real_t dy, real_t dz)
{
	_p[0] += dx;
	_p[1] += dy;
	_p[2] += dz;
	return *this;
}


inline real_t Point3::distance(const Point3 &p, const Point3 &q)
{
	return (p - q).length();
}


inline real_t Point3::squared_distance(const Point3 &p, const Point3 &q)
{
	return (p - q).squared_length();
}


inline bool Point3::collinear(const Point3 &A,
			      const Point3 &B,
			      const Point3 &C)
{
	return (B - A).cross(C - A).is_zero();
}


inline Vector3 Point3::normal(const Point3 &A,
			      const Point3 &B,
			      const Point3 &C)
{
	//assert(!collinear(A, B, C));
	return (B - A).cross(C - A).normalize();
}


inline Point3 Point3::midpoint(const Point3 &A, const Point3 &B)
{
	return Point3((A.x() + B.x()) / 2.0,
		      (A.y() + B.y()) / 2.0,
		      (A.z() + B.z()) / 2.0);
}


inline Point3 Point3::centroid(const Point3 &A,
			       const Point3 &B,
			       const Point3 &C)
{
	return Point3((A.x() + B.x() + C.x()) / 3.0,
		      (A.y() + B.y() + C.y()) / 3.0,
		      (A.z() + B.z() + C.z()) / 3.0);
}


inline Point3 centroid(const Point3 &A,
		       const Point3 &B,
		       const Point3 &C,
		       const Point3 &D)
{
	return Point3((A.x() + B.x() + C.x() + D.x()) / 4.0,
		      (A.y() + B.y() + C.y() + D.y()) / 4.0,
		      (A.z() + B.z() + C.z() + D.z()) / 4.0);
}


inline void Point3::load() const
{
#ifdef REAL_IS_FLOAT
	glVertex3f(_p[0], _p[1], _p[2]);
#else
	glVertex3d(_p[0], _p[1], _p[2]);
#endif
}


inline Point3 &Point3::transform(const Matrix4 &m)
{
	return *this *= m;
}


inline Point3 &Point3::affine_transform(const Matrix4 &m)
{
	real_t px, py, pz;
	px = x() * m[0][0] + y() * m[0][1] + z() * m[0][2] + m[0][3];
	py = x() * m[1][0] + y() * m[1][1] + z() * m[1][2] + m[1][3];
	pz = x() * m[2][0] + y() * m[2][1] + z() * m[2][2] + m[2][3];
	return reset(px, py, pz);
}

#if 0
inline void Point3::read(FILE *fp)
{
#ifdef REAL_IS_FLOAT
	read_float(&_p[0], fp);
	read_float(&_p[1], fp);
	read_float(&_p[2], fp);
#else
	float px, py, pz;
	read_float(&px, fp);
	read_float(&py, fp);
	read_float(&pz, fp);
	reset(px, py, pz);
#endif
}


inline void Point3::write(FILE *fp) const
{
	write_float(_p[0], fp);
	write_float(_p[1], fp);
	write_float(_p[2], fp);
}
#endif

inline std::istream &operator>>(std::istream &s, Point3 &p)
{
        return s >> p._p[0] >> p._p[1] >> p._p[2];
}


inline std::ostream &operator<<(std::ostream &s, const Point3 &p)
{
	return s << p._p[0] << ' ' << p._p[1] << ' ' << p._p[2];
}


END_GTBMOUSENS
