#include "gtbmouse.h"
#ifndef WIN32
#include <gtb/graphics/ray3.hpp>
#include <gtb/graphics/polygon3.hpp>
#include <gtb/graphics/box3.hpp>
#endif // WIN32


#ifdef OUTLINE
#define inline
#include <gtb/graphics/ray3.ipp>
#undef inline
#endif


using namespace std;


BEGIN_GTBMOUSENS


const Ray3 RAY3_POSITIVE_X(Point3(0.0, 0.0, 0.0), Vector3(1.0, 0.0, 0.0));
const Ray3 RAY3_NEGATIVE_X(Point3(0.0, 0.0, 0.0), Vector3(-1.0, 0.0, 0.0));
const Ray3 RAY3_POSITIVE_Y(Point3(0.0, 0.0, 0.0), Vector3(0.0, 1.0, 0.0));
const Ray3 RAY3_NEGATIVE_Y(Point3(0.0, 0.0, 0.0), Vector3(0.0, -1.0, 0.0));
const Ray3 RAY3_POSITIVE_Z(Point3(0.0, 0.0, 0.0), Vector3(0.0, 0.0, 1.0));
const Ray3 RAY3_NEGATIVE_Z(Point3(0.0, 0.0, 0.0), Vector3(0.0, 0.0, -1.0));


Ray3::Ray3()
	: _source(Point3(0.0, 0.0, 0.0)),
	  _direction(Vector3(1.0, 0.0, 0.0))
{
}


Ray3::Ray3(const Ray3 &r)
	: _source(r._source),
	  _direction(r._direction)
{
}


Ray3::Ray3(const Point3 &p, const Vector3 &d)
	: _source(p),
	  _direction(d)
{
	_direction.normalize();
}


Ray3::Ray3(const Point3 &p, const Point3 &q)
	: _source(p),
	  _direction(q - p)
{
	_direction.normalize();
}


#if 0
bool Ray3::intersects(const Polygon3 &poly) const
{
	real_t hit_time;
	Point3 hit_point;
	return intersects(poly, hit_time, hit_point);
}


bool Ray3::intersects(const Polygon3 &poly, real_t &hit_time) const
{
	Point3 hit_point;
	return intersects(poly, hit_time, hit_point);
}


bool Ray3::intersects(const Polygon3 &poly,
		      real_t &hit_time,
		      Point3 &hit_point) const
{
	const Vector3 &d = _direction;
	const Point3 &s = _source;
	real_t D = poly.D();
	const Vector3 &n = poly.normal();

	real_t nd = n * d;
	if (real::is_zero(nd)) {
		return false;
	}
	real_t ns = n * s;
	hit_time = -(D + ns) / nd;
	if (real::is_negative(hit_time)) {
		return false;
	}
	hit_point = point(hit_time);
	return poly.contains(hit_point);
}


bool Ray3::intersects(const Box3 &box) const
{
	real_t t1, t2;
	return intersects(box, t1, t2);
}
#endif


END_GTBMOUSENS
