#ifndef GTB_VECTOR2_INCLUDED
#define GTB_VECTOR2_INCLUDED

#include "gtbmousecommon.h"

BEGIN_GTBMOUSENS


class Point2;
class Vector3;


class Vector2 {
public:
	Vector2();
	Vector2(const Vector2 &v);
	Vector2(real_t vx, real_t vy);
	Vector2(const real_t v[2]);
	Vector2 &operator=(const Vector2 &v);

	real_t x() const;
	real_t y() const;

	void set_x(real_t vx);
	void set_y(real_t vy);
	Vector2 &reset(real_t vx, real_t vy);

	real_t operator[](unsigned i) const;
	real_t &operator[](unsigned i);

	bool operator==(const Vector2 &v) const;
	bool operator!=(const Vector2 &v) const;
	bool is_zero() const;

	Vector2 operator-() const;
	Vector2 &operator+=(const Vector2 &v);
	Vector2 &operator-=(const Vector2 &v);
	Vector2 &operator*=(real_t a);
	Vector2 &operator/=(real_t a);

	friend Vector2 operator+(const Vector2 &u, const Vector2 &v);
	friend Point2 operator+(const Point2 &p, const Vector2 &v);
	friend Vector2 operator-(const Vector2 &u, const Vector2 &v);
	friend Point2 operator-(const Point2 &p, const Vector2 &v);
	friend Vector2 operator*(real_t a, const Vector2 &v);
	friend Vector2 operator*(const Vector2 &v, real_t a);
	friend Vector2 operator/(const Vector2 &v, real_t a);

	real_t dot(const Vector2 &v) const;
	Vector3 cross(const Vector2 &v) const;
	real_t length() const;
	real_t squared_length() const;
	Vector2 &normalize();
	bool is_normalized() const;
	Vector2 &scale(real_t a);
	Vector2 &rotate(real_t theta);

	friend class Point2;
  	friend std::istream &operator>>(std::istream &s, Vector2 &v);
  	friend std::ostream &operator<<(std::ostream &s, const Vector2 &v);

protected:
	real_t _v[2];
};


extern const Vector2 VECTOR2_ZERO;
extern const Vector2 VECTOR2_POSITIVE_X;
extern const Vector2 VECTOR2_NEGATIVE_X;
extern const Vector2 VECTOR2_POSITIVE_Y;
extern const Vector2 VECTOR2_NEGATIVE_Y;


END_GTBMOUSENS

#ifndef OUTLINE
#include "vector2.ipp"
#endif

#endif // GTB_VECTOR2_INCLUDED
