#ifndef GTB_VIEW_INCLUDED
#define GTB_VIEW_INCLUDED

#include "gtbmousecommon.h"
#include "camera.hpp"
#include "viewport.hpp"
#include "ray3.hpp"


BEGIN_GTBMOUSENS


class View {
public:
	View();
	View(const View &view);
	View(const Camera &cam, const Viewport &vp);
	View &operator=(const View &view);

	const Camera &camera() const;
	Camera &camera();
	const Viewport &viewport() const;
	Viewport &viewport();

	void set_camera(const Camera &cam);
	void set_viewport(const Viewport &vp);

	void rotate_world(const Point3 &origin, int dx, int dy);
	void translate_world(const Point3 &origin, int dx, int dy);
	void scale_world(const Point3 &origin, int dx, int dy);
	void scale_world(real_t factor);

	void load_camera() const;
	void load_camera(const std::vector<Light> &headlights) const;
	void init_exterior_view(const Box3 &box);
	void init_interior_view(const Box3 &box);
	void init_image_view(unsigned width, unsigned height);

	Ray3 world_ray(int x, int y) const; // (0,0) is lower left corner
	Point3 viewport_point(int x, int y) const;
	Point2 viewport_point(const Point3 &p) const;

protected:
	Camera _camera;
	Viewport _viewport;
};


END_GTBMOUSENS

#ifndef OUTLINE
#include "view.ipp"
#endif

#endif // GTB_VIEW_INCLUDED
